﻿Option Strict On
Option Explicit On
Public Class Asset
    Inherits Panel

    ''' <summary>
    ''' Gets or sets the depth of the asset
    ''' </summary>
    ''' <returns>Value between 0-255</returns>
    Public Property Depth As Byte

    ''' <summary>
    ''' Gets or sets the length of the asset
    ''' </summary>
    ''' <returns></returns>
    Public Property Length As UInteger

    ''' <summary>
    ''' Gets or sets the identifying offical number of the asset
    ''' </summary>
    ''' <returns>String</returns>
    Public Property OfficalNumber As String

    ''' <summary>
    ''' Gets or sets the weight of the asset
    ''' </summary>
    ''' <returns>Value between 0-4294967295</returns>
    Public Property Weight As UInteger

    ''' <summary>
    ''' Gets or sets the year that the asset was built
    ''' </summary>
    ''' <returns>Value between 0-4294967295</returns>
    Public Property YearBuilt As UInteger

    Sub New()
        With Me
            .BorderStyle = BorderStyle.FixedSingle
            .Margin = New Padding(0)
            .Size = New Size(250, 100)
        End With
    End Sub

End Class

Public Class Barge
    Inherits Asset

    ''' <summary>
    ''' Gets or sets the name of the heater that the boat uses
    ''' </summary>
    ''' <returns>String</returns>
    Public Property Heater As String

    ''' <summary>
    ''' Gets or sets the width of the barge
    ''' </summary>
    ''' <returns>System.UInt32</returns>
    Public Shadows Property Width As UInteger

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)

        Using brushText As Brush = New SolidBrush(Color.Black)
            Dim regularText As String = $"Official #: {Me.OfficalNumber} / {Me.Length} x {Me.Width} x {Me.Depth}{Environment.NewLine}" &
                                        $"{Me.Weight} BBL / Built: {Me.YearBuilt}{Environment.NewLine}" &
                                        $"{Me.Heater} / Built: {Me.YearBuilt}"

            'Set the height of the Panel to the sum of the heigth of the two Strings
            MyBase.Height = TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Height + TextRenderer.MeasureText(regularText, Me.Font).Height + 5

            'Set the width of the Panel to the greater of the two Strings
            Dim boldWidth As Integer = TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Width
            Dim regularWidth As Integer = TextRenderer.MeasureText(regularText, Me.Font).Width
            MyBase.Width = If(boldWidth > regularWidth, boldWidth, regularWidth)

            'Draw the bolded text
            e.Graphics.DrawString(Me.Name, New Font(Me.Font, FontStyle.Bold), brushText, Me.DisplayRectangle, New StringFormat() With {.Alignment = StringAlignment.Center})

            'Draw the rest of the text
            e.Graphics.DrawString(regularText, Me.Font, brushText, New Rectangle(0, TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Height + 2, Me.DisplayRectangle.Width, Me.DisplayRectangle.Height), New StringFormat() With {.Alignment = StringAlignment.Center})

        End Using
    End Sub

End Class

Public Class Boat
    Inherits Asset

    ''' <summary>
    ''' Gets or sets the bredth of the boat
    ''' </summary>
    ''' <returns>System.UInt32</returns>
    Public Property Breadth As UInteger

    ''' <summary>
    ''' Gets or sets the name of the engine that the barge uses
    ''' </summary>
    ''' <returns>String</returns>
    Public Property Engine As String

    ''' <summary>
    ''' Gets or sets the total horsepower of the engine
    ''' </summary>
    ''' <returns>Value between 0-4294967295</returns>
    Public Property Horsepower As UInteger

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)

        Using brushText As Brush = New SolidBrush(Color.Black)
            'The text that is not bold
            Dim regularText As String = $"Official #: {Me.OfficalNumber} / {Me.Length} x {Me.Breadth} x {Me.Depth}{Environment.NewLine}" &
                                  $"Gross Tons: {Me.Weight} / HP: {Me.Horsepower}{Environment.NewLine}" &
                                  $"{Me.Engine} / Built: {Me.YearBuilt}"

            'Set the height of the Panel to the sum of the heigth of the two Strings
            MyBase.Height = TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Height + TextRenderer.MeasureText(regularText, Me.Font).Height + 5

            'Set the width of the Panel to the greater of the two Strings
            Dim boldWidth As Integer = TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Width
            Dim regularWidth As Integer = TextRenderer.MeasureText(regularText, Me.Font).Width
            MyBase.Width = If(boldWidth > regularWidth, boldWidth, regularWidth)

            'Draw the bolded text
            e.Graphics.DrawString(Me.Name, New Font(Me.Font, FontStyle.Bold), brushText, Me.DisplayRectangle, New StringFormat() With {.Alignment = StringAlignment.Center})

            'Draw the rest of the text
            e.Graphics.DrawString(regularText, Me.Font, brushText, New Rectangle(0, TextRenderer.MeasureText(Me.Name, New Font(Me.Font, FontStyle.Bold)).Height + 2, Me.DisplayRectangle.Width, Me.DisplayRectangle.Height), New StringFormat() With {.Alignment = StringAlignment.Center})

        End Using
    End Sub

End Class